/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <atompub/atom.h>

#include <glib.h>

#include <string.h>

static
void atom_get (AtomCtx *ctx, AtomRequest *request)
{
  if (atom_is_feed (ctx, atom_request_request (request)))
    {
      AtomFeed *feed;
      feed = atom_retrieve_feed (ctx);
      if (feed != NULL)
        {
          atom_handle_feed (ctx, feed);
          atom_feed_delete (feed);
        }
    }
  else
     {
       AtomEntry *entry;
       entry = atom_retrieve_entry (ctx, atom_request_request (request));
       if (entry != NULL)
         {
           atom_handle_entry (ctx, entry);
           atom_entry_delete (entry);
         }
     }
}

static void
atom_post (AtomCtx *ctx, AtomRequest *request)
{
  AtomError *error;
  /* Publish requests go to the same IRI as the feed IRI */
  if (!atom_is_feed (ctx, atom_request_request (request)))
    {
      error = atom_error_new ();
      atom_error_code_set (error, 400);
      atom_error_message_set (error, "Bad Request");
      atom_error_set (ctx, error);
    }
  else
    {
      char *content = NULL;
      size_t len = 0;
      AtomEntry *entry;
      atom_request_content (request, &content, &len);
      /* TODO: If the content is not an Atom Entry, consider it a media
       * publish request */
      entry = atom_entry_new_data_len (content, len);
      if (entry == NULL)
        {
          error = atom_error_new ();
          atom_error_code_set (error, 400);
          atom_error_message_set (error, "Bad Request");
          atom_error_set (ctx, error);
        }
      else
        {
          atom_publish_entry (ctx, atom_request_request (request), entry);
          /* TODO: Entry should be the backend representation and more
           * information should be given to the frontend */
          if (!atom_error_get (ctx))
            atom_handle_publish (ctx, entry);
        }
    }
}

void
atom_serve_request (AtomCtx *ctx)
{
  AtomRequest *request;
  AtomError *error;
  request = atom_get_request (ctx);
  if (request != NULL)
    {
      int type = atom_request_type (request);
      switch (type)
        {
        case ATOM_REQUEST_GET:
          atom_get (ctx, request);
          break;
        case ATOM_REQUEST_POST:
          atom_post (ctx, request);
          break;
        default:
          error = atom_error_new ();
          atom_error_code_set (error, 501);
          atom_error_message_set (error, "Not Implemented");
          atom_error_set (ctx, error);
        }
    }
  if (atom_error_get (ctx))
    {
      atom_handle_error (ctx);
      atom_error_set (ctx, NULL);
    }
}
